// AnimInfoBar.cpp : implementation file
//

#include "stdafx.h"
#include "modeler.h"
#include "AnimInfoBar.h"
#include "ChildFrm.h"
#include "MainFrm.h"
#include "MODELERDoc.h"
#include "MODELERView.h"
#include "FrameSpeedDialog.h"
#include "AnimationNameDialog.h"

#include <windowsx.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnimInfoBar

CAnimInfoBar::CAnimInfoBar() : m_psrCurrent(NULL),
	m_pndSelected(NULL)
{
	m_ilAnimInfoImgList.Create(IDB_ANIMTREECTRL_0, 16, 0, RGB (255, 0, 255));
}

CAnimInfoBar::~CAnimInfoBar()
{
}


BEGIN_MESSAGE_MAP(CAnimInfoBar, CBCGSizingControlBar)
	//{{AFX_MSG_MAP(CAnimInfoBar)
	ON_WM_CREATE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CAnimInfoBar message handlers

int CAnimInfoBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CBCGSizingControlBar::OnCreate(lpCreateStruct) == -1)
		return -1;

	CRect rectDummy;
	rectDummy.SetRectEmpty ();

	if ( !m_wndAnimInfo.Create(WS_CHILD | WS_VISIBLE | TVS_SHOWSELALWAYS | TVS_HASLINES | TVS_LINESATROOT | TVS_DISABLEDRAGDROP | TVS_HASBUTTONS, rectDummy, this, 2) )
	{
		TRACE0("Failed to create animation info tree control\n");
		return -1;      // fail to create
	}

	if ( !m_wndMessage.Create("No animation info available", WS_CHILD | WS_VISIBLE | SS_CENTER, rectDummy, this) )
	{
		TRACE0("Failed to create static control\n");
		return -1;      // fail to create
	}

	m_wndAnimInfo.ModifyStyleEx(0, WS_EX_CLIENTEDGE);

	m_wndAnimInfo.SetFont(&globalData.fontRegular,FALSE);
	m_wndAnimInfo.SetImageList(&m_ilAnimInfoImgList, TVSIL_NORMAL);

	CToolTipCtrl		*pwndToolTip = m_wndAnimInfo.GetToolTips();

	if ( pwndToolTip != NULL )
	{
		pwndToolTip->SetWindowPos(&wndTopMost, -1, -1, -1, -1, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
	}

	m_wndMessage.SetFont(&globalData.fontUnderline,FALSE);

	// *** imposta lo stato di visibilit ***

	SetChildrenVisibilityState();

	return 0;
}

void CAnimInfoBar::OnSize(UINT nType, int cx, int cy) 
{
	CBCGSizingControlBar::OnSize(nType, cx, cy);

	if ( m_wndAnimInfo.m_hWnd != NULL && m_wndMessage.m_hWnd != NULL )
	{
		m_wndAnimInfo.SetWindowPos(NULL, 0, 0, cx, cy, SWP_NOACTIVATE | SWP_NOZORDER);

		int			iY = cy / 3;

		m_wndMessage.SetWindowPos(NULL, 0, iY, cx, cy - iY, SWP_NOACTIVATE | SWP_NOZORDER);

		if ( m_wndAnimInfo.GetStyle () & WS_VISIBLE ) m_wndAnimInfo.RedrawWindow();
		if ( m_wndMessage.GetStyle () & WS_VISIBLE ) m_wndMessage.RedrawWindow();
	}
}

void CAnimInfoBar::OnUpdateCmdUI(CFrameWnd* pTarget, BOOL /*bDisableIfNoHndler*/)
{
	CNode			*pndOldSelected = m_pndSelected;

	m_pndSelected = NULL;

	// *** prosegue solo se la control bar  visibile ***

	if ( GetStyle() & WS_VISIBLE )
	{

		// *** ottiene la view ed il documento attivi ***

		CSceneRoot		*psrCurrent = NULL;

		CMainFrame		*pMainFrame = DYNAMIC_DOWNCAST(CMainFrame, pTarget);

		ASSERT( pMainFrame != NULL );

		BOOL			bMaximized;

		CChildFrame		*pChildFrame = (CChildFrame*)pMainFrame->MDIGetActive(&bMaximized);

		if ( pChildFrame != NULL )
		{
			CMODELERView	*pView = (CMODELERView*)pChildFrame->GetActiveView();

			ASSERT( pView != NULL );

			CMODELERDoc		*pDocument = pView->GetDocument();

			ASSERT( pDocument != NULL );

			if ( pView->m_iShownAnimation != -1 )
			{
				psrCurrent = pDocument->m_vAnimations[pView->m_iShownAnimation];
			}
		}

		// *** valuta se si tratta di una nuova animazione ***

		if ( psrCurrent != m_psrCurrent )
		{
			LockWindowUpdate();

			m_psrCurrent = psrCurrent;

			if ( m_psrCurrent ) m_psrCurrent->SetTreeCtrlWithInfo(m_wndAnimInfo.m_hWnd);

			SetChildrenVisibilityState();

			UnlockWindowUpdate();
		}

		// *** imposta m_pndSelected ***

		if ( m_wndAnimInfo.GetStyle() & WS_VISIBLE )
		{

			HTREEITEM		hSelected = m_wndAnimInfo.GetSelectedItem();

			if ( hSelected != NULL )
			{
				m_pndSelected = (CNode*)m_wndAnimInfo.GetItemData(hSelected);
			}

		}

		// *** valuta se aggiornare ***

		if ( m_pndSelected != pndOldSelected )
		{
			if ( pChildFrame != NULL )
			{
				vector<HWND>	vhwndUpdatedViews;

				CMODELERView	*pView = (CMODELERView*)pChildFrame->GetActiveView();

				ASSERT( pView != NULL );

				CMODELERDoc		*pDocument = pView->GetDocument();

				ASSERT( pDocument != NULL );

				POSITION viewpos = pDocument->GetFirstViewPosition();
				while (viewpos != NULL)
				{
					CMODELERView* pModelerView = (CMODELERView*)pDocument->GetNextView(viewpos);

					if ( pModelerView->m_iShownAnimation == pView->m_iShownAnimation )
					{
						pModelerView->RedrawWindow();
						vhwndUpdatedViews.push_back(pModelerView->m_hWnd);
					}
				}

				// *** aggiorna le views precedenti ***

				for ( int i=0; i<m_vhwndUpdatedViews.size(); i++ )
				{
					for ( int j=0; j<vhwndUpdatedViews.size(); j++ )
						if ( vhwndUpdatedViews[j] == m_vhwndUpdatedViews[i] )
							break;

					if ( j==vhwndUpdatedViews.size() && ::IsWindow(m_vhwndUpdatedViews[i]) )
					{
						::RedrawWindow(m_vhwndUpdatedViews[i], NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);
					}
				}

				// *** copia vhwndUpdatedViews in m_vhwndUpdatedViews ***

				m_vhwndUpdatedViews = vhwndUpdatedViews;

			}
			else
			{
				m_vhwndUpdatedViews.clear();
			}
		}

	}
	else	// la control bar non  visibile
	{
		for ( int i=0; i<m_vhwndUpdatedViews.size(); i++ )
		{
			if ( ::IsWindow(m_vhwndUpdatedViews[i]) )
			{
				::RedrawWindow(m_vhwndUpdatedViews[i], NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);
			}
		}

		m_vhwndUpdatedViews.clear();
	}
}

BOOL CAnimInfoBar::PreTranslateMessage(MSG* pMsg) 
{
	if ( pMsg->hwnd == m_wndAnimInfo.m_hWnd )
	{
		if ( pMsg->message == WM_RBUTTONDOWN )
		{
			pMsg->message = WM_LBUTTONDOWN;
		}
		else if ( pMsg->message == WM_RBUTTONUP )
		{
			pMsg->message = WM_LBUTTONUP;
		}
		else if ( pMsg->message == WM_LBUTTONDBLCLK || pMsg->message == WM_RBUTTONDBLCLK || ( pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_RETURN ) )
		{
			HTREEITEM	htiItem;

			if ( pMsg->message == WM_LBUTTONDBLCLK || pMsg->message == WM_RBUTTONDBLCLK )
			{
				CPoint		pntCursor(GET_X_LPARAM(pMsg->lParam), GET_Y_LPARAM(pMsg->lParam));

				htiItem = m_wndAnimInfo.HitTest(pntCursor);
			}
			else
			{
				htiItem = m_wndAnimInfo.GetSelectedItem();
			}

			if ( htiItem != NULL )
			{

				CNode		*pndNode = (CNode*)m_wndAnimInfo.GetItemData(htiItem);

				if ( pndNode != NULL )
				{



					m_wndAnimInfo.SetFocus();
					return TRUE;
				}
				else
				{
					CString		strItemName = m_wndAnimInfo.GetItemText(htiItem);

					if ( strItemName.Left(15) == "Animation Name:" )
					{
						ASSERT( m_psrCurrent != NULL );

						CAnimationNameDialog		wndDialog(m_psrCurrent);

						if ( wndDialog.DoModal() == IDOK )
						{
							strItemName.Format("Animation Name: \"%s\"", m_psrCurrent->m_pASD->AnimName);
							m_wndAnimInfo.SetItemText(htiItem, strItemName);
						}

						m_wndAnimInfo.SetFocus();
						return TRUE;
					}
					else if ( strItemName.Left(12) == "Frame Speed:" )
					{
						ASSERT( m_psrCurrent != NULL );

						CFrameSpeedDialog			wndDialog(m_psrCurrent);

						if ( wndDialog.DoModal() == IDOK )
						{

							// *** aggiorna l'item del tree control ***

							strItemName.Format("Frame Speed: %i fps", m_psrCurrent->m_pGI->frame_speed);
							m_wndAnimInfo.SetItemText(htiItem, strItemName);

							// *** valuta se ci sono view che stanno facendo girare l'animazione ***

							CWnd			*pMainWnd = AfxGetApp()->m_pMainWnd;

							ASSERT( pMainWnd != NULL );

							CMainFrame		*pMainFrame = DYNAMIC_DOWNCAST(CMainFrame, pMainWnd);

							ASSERT( pMainFrame != NULL );

							BOOL			bMaximized;

							CChildFrame		*pChildFrame = (CChildFrame*)pMainFrame->MDIGetActive(&bMaximized);

							if ( pChildFrame != NULL )
							{
								CMODELERView	*pView = (CMODELERView*)pChildFrame->GetActiveView();

								ASSERT( pView != NULL );

								CMODELERDoc		*pDocument = pView->GetDocument();

								ASSERT( pDocument != NULL );

								POSITION viewpos = pDocument->GetFirstViewPosition();
								while (viewpos != NULL)
								{
									CMODELERView* pModelerView = (CMODELERView*)pDocument->GetNextView(viewpos);

									if ( pModelerView->m_iShownAnimation == pView->m_iShownAnimation &&
										pModelerView->m_bIsPlaying )
									{
										pModelerView->PlayPauseAnimation();
										pModelerView->PlayPauseAnimation();
									}
								}

							}

						}

						m_wndAnimInfo.SetFocus();
						return TRUE;
					}
					else if ( m_wndAnimInfo.ItemHasChildren(htiItem) == FALSE )
					{
						AfxMessageBox("Selected item is not editable");

						m_wndAnimInfo.SetFocus();
						return TRUE;
					}
					else if ( pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_RETURN )
					{
						m_wndAnimInfo.Expand(htiItem, TVE_TOGGLE);
					}
				}

			}
		}
	}

	return CBCGSizingControlBar::PreTranslateMessage(pMsg);
}
