// AnimOpenGLRenderer.cpp : implementation file
//

#include "stdafx.h"
#include "MODELER.h"
#include "AnimOpenGLRenderer.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnimOpenGLRenderer

HGLRC	CAnimOpenGLRenderer::first_hglrc = NULL;
int		CAnimOpenGLRenderer::font_list;

CAnimOpenGLRenderer::CAnimOpenGLRenderer() :
	m_pSceneRoot(NULL),
	m_fRotX(35), m_fRotY(0), m_fRotZ(45),
	m_fPosX(-144), m_fPosY(-144), m_fPosZ(144),
	m_eRenderMode(CNode::eSolid),
	m_tTime(0),
	m_pntAbsoluteCursorPos(0,0),
	m_bShowFrameNumbers(TRUE)
{
	SetDefaultBackgroundColor();

	m_fAxesColors[0][0]=255.0f/255.0f;
	m_fAxesColors[0][1]=0.0f/255.0f;
	m_fAxesColors[0][2]=0.0f/255.0f;
	m_fAxesColors[1][0]=0.0f/255.0f;
	m_fAxesColors[1][1]=255.0f/255.0f;
	m_fAxesColors[1][2]=0.0f/255.0f;
	m_fAxesColors[2][0]=0.0f/255.0f;
	m_fAxesColors[2][1]=0.0f/255.0f;
	m_fAxesColors[2][2]=255.0f/255.0f;

	m_fAmbient[0] = 0.0f;
	m_fAmbient[1] = 0.0f;
	m_fAmbient[2] = 0.0f;
	m_fAmbient[3] = 1.0f;

	m_fDiffuse[0] = 0.3f;
	m_fDiffuse[1] = 0.6f;
	m_fDiffuse[2] = 0.9f;
	m_fDiffuse[3] = 1.0f;
}

CAnimOpenGLRenderer::~CAnimOpenGLRenderer()
{
}


BEGIN_MESSAGE_MAP(CAnimOpenGLRenderer, CWnd)
	//{{AFX_MSG_MAP(CAnimOpenGLRenderer)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	//}}AFX_MSG_MAP
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CAnimOpenGLRenderer message handlers

int CAnimOpenGLRenderer::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	// *** inizializzazione OpenGL ***

	CClientDC	dc(this);

	int pixelformat=SetupPixelFormat(dc.GetSafeHdc());
	if ( pixelformat==0 ) {
		AfxMessageBox("SetupPixelFormat failed!");
		return -1;
	}

	hglrc=wglCreateContext(dc.GetSafeHdc());
	if ( hglrc==0 ) {
		AfxMessageBox("wglCreateContext failed!");
		return -1;
	}

	if ( first_hglrc!=NULL ) {

		// *** wglShareLists ***

		if ( wglShareLists(first_hglrc,hglrc)==0 ) {
			AfxMessageBox("wglShareLists failed!");
			return -1;
		}

	}
	else {

		// *** crea il font, se si tratta della prima view creata... ***

		if ( wglMakeCurrent(dc.GetSafeHdc(),hglrc)==0 ) {
			AfxMessageBox("wglMakeCurrent failed!");
			return -1;
		}

		CFont	*OldFont=dc.SelectObject(&globalData.fontRegular);

		font_list=glGenLists(256);
		if ( font_list==0 ) {
			AfxMessageBox("glGenLists failed!");
			return -1;
		}

		if ( !wglUseFontBitmaps(dc.GetSafeHdc(), 1, 255, font_list) ) {
			AfxMessageBox("wglUseFontBitmaps failed!");
			return -1;
		}

		dc.SelectObject(OldFont);

		// imposta first_hglrc per riutilizzarlo per le view successive
		first_hglrc=hglrc;

	}

	// *** disattiva il rendering context attivo ***

	wglMakeCurrent( NULL, NULL );

	return 0;
}

void CAnimOpenGLRenderer::OnDestroy() 
{
	CWnd::OnDestroy();
	
	// TODO: Add your message handler code here
	
	// *** rilascia le OpenGL ***
	
	wglMakeCurrent( NULL, NULL );
	wglDeleteContext( hglrc );
}

int CAnimOpenGLRenderer::SetupPixelFormat (HDC hDC)
{
	PIXELFORMATDESCRIPTOR pfd={
		sizeof(PIXELFORMATDESCRIPTOR),	// size of this pfd
		1,								// version number
		PFD_DRAW_TO_WINDOW |			// support window
		PFD_SUPPORT_OPENGL |			// support OpenGL
		PFD_DOUBLEBUFFER,				// double buffered
		PFD_TYPE_RGBA,					// RGBA type
		24,								// 24-bit color depth
		0, 0, 0, 0, 0, 0,				// color bits ignored
		0,								// no alpha buffer
		0,								// shift bit ignored
		0,								// no accumulation buffer
		0, 0, 0, 0,						// accum bits ignored
		32,							    // depth bits
		0,								// no stencil buffer
		0,								// no auxiliary buffer
		PFD_MAIN_PLANE,					// main layer
		0,								// reserved
		0, 0, 0							// layer masks ignored
    };
    int pixelformat=0;

	pixelformat=ChoosePixelFormat(hDC, &pfd);
	if ( pixelformat==0 )
		return(0);

    if (!SetPixelFormat(hDC, pixelformat, &pfd))
		return(0);

	return(pixelformat);
}

BOOL CAnimOpenGLRenderer::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class

	cs.style |= WS_CLIPCHILDREN | WS_CLIPSIBLINGS;

	return CWnd::PreCreateWindow(cs);
}

void CAnimOpenGLRenderer::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here

	if ( wglMakeCurrent(dc.GetSafeHdc(),hglrc)==0 ) {
		AfxMessageBox("wglMakeCurrent failed!");
		AfxAbort();
	}

	RECT		viewRect;
	GetClientRect(&viewRect);

	glViewport(0,0,viewRect.right,viewRect.bottom);
	glScissor(0,0,viewRect.right,viewRect.bottom);
	glEnable(GL_SCISSOR_TEST);
	glClearColor(m_fBackgroundColor[0],
		m_fBackgroundColor[1],
		m_fBackgroundColor[2],0);
	glClear(GL_COLOR_BUFFER_BIT);

	if ( m_pSceneRoot != NULL )
	{

		glClearDepth(1.0f);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		float aspect=(float)viewRect.right/viewRect.bottom;
		gluPerspective(2*atan(1.0f/aspect)*180/PI,aspect,2,9000);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glRotatef(-90,1,0,0);
		glRotatef(m_fRotY,0,1,0);
		glRotatef(m_fRotX,1,0,0);
		glRotatef(m_fRotZ,0,0,1);
		glTranslatef(-m_fPosX,-m_fPosY,-m_fPosZ);

		if ( m_eRenderMode==CNode::eWireframe )
		{
			glClear(GL_DEPTH_BUFFER_BIT);

			glDepthFunc(GL_LEQUAL);
			glEnable(GL_DEPTH_TEST);

			// *** disegna la griglia ***
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_TEXTURE_1D);
			glDisable(GL_BLEND);
			glDisable(GL_CULL_FACE);
			DrawGrid(1000);
			// *** ***

			glShadeModel(GL_FLAT);
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

			glCullFace(GL_BACK);
			glEnable(GL_CULL_FACE);

			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			glDisable(GL_TEXTURE_1D);

			glDisable(GL_LIGHTING);
			glDisable(GL_NORMALIZE);
		}
		else if ( m_eRenderMode==CNode::eSolid || m_eRenderMode==CNode::eSolidPlusWireframe )
		{
			glClear(GL_DEPTH_BUFFER_BIT);

			glDepthFunc(GL_LEQUAL);
			glEnable(GL_DEPTH_TEST);

			// *** disegna la griglia ***
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_TEXTURE_1D);
			glDisable(GL_BLEND);
			glDisable(GL_CULL_FACE);
			DrawGrid(1000);
			// *** ***

			glShadeModel(GL_FLAT);
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

			glCullFace(GL_BACK);
			glEnable(GL_CULL_FACE);

			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			glDisable(GL_TEXTURE_1D);

			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			glEnable(GL_NORMALIZE);

			glLightfv(GL_LIGHT0, GL_AMBIENT, m_fAmbient);
			glLightfv(GL_LIGHT0, GL_DIFFUSE, m_fDiffuse);
		}
		else if ( m_eRenderMode==CNode::eTexture )
		{
		}

		m_pSceneRoot->EvalWorldState(m_tTime);
		glColor3f(1,1,1);
		m_pSceneRoot->Render(m_eRenderMode==CNode::eSolidPlusWireframe ? CNode::eSolid : m_eRenderMode);

		// disattiva alcune impostazioni settate precedentemente
		glDisable(GL_LIGHTING);
		glDisable(GL_NORMALIZE);

		// completa eSolidPlusWireframe
		if ( m_eRenderMode==CNode::eSolidPlusWireframe )
		{
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			m_pSceneRoot->Render(CNode::eWireframe);
		}

		// valuta se evidenziare un nodo
		CNode		*pndSelected = ((CMainFrame*)AfxGetApp()->m_pMainWnd)->m_wndAnimInfoBar.m_pndSelected;

		if ( pndSelected != NULL && m_pSceneRoot->IsChild(pndSelected) )
		{
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			pndSelected->Render(CNode::eWireframeSpecialGreen);
		}

		// disegna gli assi di riferimento (GL_PROJECTION e GL_MODELVIEW modificate!)
		if ( 1 ) {

			RECT	rect;
			GetWindowRect(&rect);

			glDisable(GL_TEXTURE_2D);
			glDisable(GL_TEXTURE_1D);
			glDisable(GL_DEPTH_TEST);
			glDisable(GL_BLEND);
			glDisable(GL_CULL_FACE);
			DrawMiniAxes(&rect);

			// *** stampa il numero di frame corrente ***

			if ( m_bShowFrameNumbers )
			{

				char		szString[MAX_PATH];

				sprintf(szString, "%i / %i", m_tTime / m_pSceneRoot->m_pGI->ticks_per_frame,
					m_pSceneRoot->m_pGI->last_frame - m_pSceneRoot->m_pGI->first_frame);

				glColor3f(229.0f/255.0f, 229.0f/255.0f, 229.0f/255.0f);

				glRasterPos2f(rect.left + 3, rect.top + 11);
				glCallLists(strlen(szString),GL_UNSIGNED_BYTE,szString);

			}

		}

	}

	SwapBuffers(dc.GetSafeHdc());

	// Do not call CWnd::OnPaint() for painting messages
}

BOOL CAnimOpenGLRenderer::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	return TRUE;
}

void CAnimOpenGLRenderer::OnLButtonDown(UINT nFlags, CPoint point)
{
	if ( GetCapture() == NULL && m_pSceneRoot != NULL )
	{
		OnMouseButtonDown(point);
	}

	CWnd::OnLButtonDown(nFlags, point);
}

void CAnimOpenGLRenderer::OnLButtonUp(UINT nFlags, CPoint point)
{
	if ( GetCapture() == this )
	{
		ReleaseCapture();
	}

	CWnd::OnLButtonUp(nFlags, point);
}

void CAnimOpenGLRenderer::OnMouseMove(UINT nFlags, CPoint point)
{
	if ( GetCapture() == this )
	{
		// *** provoca lo spostamento del cursore da una parte all'altra dello schermo ***

		RECT		rect;
		GetClipCursor(&rect);

		CPoint		p=point;
		bool		changed=false;
		ClientToScreen(&p);

		// si occupa di m_pntAbsoluteCursorPos
		m_pntAbsoluteCursorPos+=p-m_pntOldCursorPos;
		m_pntOldCursorPos=p;

		if ( p.x == rect.left ) {
			p.x=rect.right-2;
			changed=true;
		}
		else if ( p.x == rect.right-1 ) {
			p.x=rect.left+1;
			changed=true;
		}

		if ( p.y == rect.top ) {
			p.y=rect.bottom-2;
			changed=true;
		}
		else if ( p.y == rect.bottom-1 ) {
			p.y=rect.top+1;
			changed=true;
		}

		if ( changed != false ) {

			// imposta m_pntOldCursorPos
			m_pntOldCursorPos=p;

			// modifica la posizione del cursore
			ShowCursor(FALSE);
			SetCursorPos(p.x,p.y);
			ShowCursor(TRUE);
			ScreenToClient(&p);
			point=p;

		}

		// *** si occupa di m_bContextMenu ***

		if ( abs(m_pntAbsoluteCursorPos.x) > 2 ||
			abs(m_pntAbsoluteCursorPos.y) > 2 )
		{
			m_bContextMenu = FALSE;
		}

		// *** naviga ***

		int			x = m_pntAbsoluteCursorPos.x;
		int			y = m_pntAbsoluteCursorPos.y;

		if ( m_bRight == FALSE )
		{

			// modifica l'angolo
			m_fRotX=m_f_op_1_xa-(float)y/3;
			m_fRotZ=m_f_op_1_za-(float)x/3;

		}
		else
		{

			// modifica la posizione
			if ( GetAsyncKeyState(VK_SHIFT) >> 15 ) {
				m_fPosX+=-m_f_op_2_g*(y-m_i_op_2_y)-m_f_op_2_d*(x-m_i_op_2_x);
				m_fPosY+=-m_f_op_2_h*(y-m_i_op_2_y)-m_f_op_2_e*(x-m_i_op_2_x);
				m_fPosZ+=-m_f_op_2_i*(y-m_i_op_2_y)-m_f_op_2_f*(x-m_i_op_2_x);
			}
			else {
				m_fPosX+=-m_f_op_2_a*(y-m_i_op_2_y)-m_f_op_2_d*(x-m_i_op_2_x);
				m_fPosY+=-m_f_op_2_b*(y-m_i_op_2_y)-m_f_op_2_e*(x-m_i_op_2_x);
				m_fPosZ+=-m_f_op_2_c*(y-m_i_op_2_y)-m_f_op_2_f*(x-m_i_op_2_x);
			}

			// imposta m_i_op_2_x e m_i_op_2_y
			m_i_op_2_x=x;
			m_i_op_2_y=y;

		}

		RedrawWindow();

	}

	CWnd::OnMouseMove(nFlags, point);
}

void CAnimOpenGLRenderer::DrawMiniAxes (RECT *r)
{
	matrix_t		matrices[2],matrix;
	vector_t		x,y,z;
	float			ox,oy;
	float			d=25;
	float			a,b;

	// *** imposta le nuove matrici per le trasformazioni ***

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity ();
	glOrtho(r->left,r->right,r->bottom,r->top,-8000,8000);

	// *** calcola la matrice e trasforma le tre normali degli assi ***

	MatrixGenerateZRotate(&matrices[0],m_fRotZ);
	MatrixGenerateXRotate(&matrices[1],m_fRotX);

	MatrixMultiply(&matrices[0],&matrices[1],&matrix);

	vector_t		vXVector = {1,0,0};
	vector_t		vYVector = {0,1,0};
	vector_t		vZVector = {0,0,1};

	MatrixApply(&matrix,&vXVector,&x);
	MatrixApply(&matrix,&vYVector,&y);
	MatrixApply(&matrix,&vZVector,&z);

	ox=r->left+(d+1);
	oy=r->bottom-(d+1);

	glBegin(GL_LINES);
	glColor3fv(m_fAxesColors[0]);
	glVertex2f(ox,oy);
	glVertex2f( a=(x.x*d + ox) , b=(-x.z*d + oy) );
	glEnd();

	glRasterPos2f(a+2,b-2);
	glCallLists(1,GL_UNSIGNED_BYTE,"X");

	glBegin(GL_LINES);
	glColor3fv(m_fAxesColors[1]);
	glVertex2f(ox,oy);
	glVertex2f( a=(y.x*d + ox) , b=(-y.z*d + oy) );
	glEnd();

	glRasterPos2f(a+2,b-2);
	glCallLists(1,GL_UNSIGNED_BYTE,"Y");

	glBegin(GL_LINES);
	glColor3fv(m_fAxesColors[2]);
	glVertex2f(ox,oy);
	glVertex2f( a=(z.x*d + ox) , b=(-z.z*d + oy) );
	glEnd();

	glRasterPos2f(a+2,b-2);
	glCallLists(1,GL_UNSIGNED_BYTE,"Z");

	return;
}

void CAnimOpenGLRenderer::OnMouseButtonDown (CPoint point, BOOL bRight /*= FALSE*/)
{
	// resetta m_pntAbsoluteCursorPos
	m_pntAbsoluteCursorPos.x=0;
	m_pntAbsoluteCursorPos.y=0;

	// converte point ed imposta m_pntOldCursorPos
	CPoint		p=point;
	ClientToScreen(&p);
	m_pntOldCursorPos=p;

	// *** calcola e salva le informazioni per l'operazione ***

	m_bRight = bRight;

	if ( bRight == FALSE )
	{
		m_f_op_1_xa=m_fRotX;
		m_f_op_1_za=m_fRotZ;
	}
	else
	{
		float mycos=(float)MathLibCos(m_fRotX);

		// resetta m_i_op_2_x e m_i_op_2_y
		m_i_op_2_x=0;
		m_i_op_2_y=0;

		// calcola seni e coseni
		m_f_op_2_a=(float)MathLibSin(m_fRotZ)*mycos;
		m_f_op_2_b=(float)MathLibCos(m_fRotZ)*mycos;
		m_f_op_2_c=(float)-MathLibSin(m_fRotX);
		m_f_op_2_d=(float)MathLibCos(m_fRotZ);
		m_f_op_2_e=(float)-MathLibSin(m_fRotZ);
		m_f_op_2_f=0.0f;
		m_f_op_2_g=0.0f;
		m_f_op_2_h=0.0f;
		m_f_op_2_i=-(float)MathLibCos(m_fRotX);
	}

	// cattura il mouse
	SetCapture();
}

void CAnimOpenGLRenderer::OnRButtonDown(UINT nFlags, CPoint point)
{
	// imposta m_bContextMenu a TRUE
	m_bContextMenu = TRUE;

	if ( GetCapture() == NULL && m_pSceneRoot != NULL )
	{
		OnMouseButtonDown(point, TRUE);
	}

	CWnd::OnRButtonDown(nFlags, point);
}

void CAnimOpenGLRenderer::OnRButtonUp(UINT nFlags, CPoint point)
{
	if ( GetCapture() == this )
	{
		ReleaseCapture();
	}

	CWnd::OnRButtonUp(nFlags, point);
}

void CAnimOpenGLRenderer::OnContextMenu(CWnd* pWnd, CPoint point)
{
	if ( m_bContextMenu )
	{
		CWnd::OnContextMenu(pWnd, point);
	}
}

void CAnimOpenGLRenderer::DrawGrid (int iExtension)
{
	int			iOffset5 = iExtension / 5;
	int			iOffset10 = iExtension / 10;

	glBegin(GL_LINES);

	for ( int x = -iExtension; x <= iExtension; x += 10 )
	{
		if ( x == 0 )
		{
			continue;
		}
		else if ( x%100 )
		{
			glColor3f(151.0f/255.0f, 151.0f/255.0f, 151.0f/255.0f);
		}
		else
		{
			glColor3f(116.0f/255.0f, 116.0f/255.0f, 116.0f/255.0f);
		}

		glVertex2i(x, iExtension);
		glVertex2i(x, -iExtension);
	}

	for ( int y = -iExtension; y <= iExtension; y += 10 )
	{
		if ( y == 0 )
		{
			continue;
		}
		else if ( y%100 )
		{
			glColor3f(151.0f/255.0f, 151.0f/255.0f, 151.0f/255.0f);
		}
		else
		{
			glColor3f(116.0f/255.0f, 116.0f/255.0f, 116.0f/255.0f);
		}

		glVertex2i(-iExtension, y);
		glVertex2i(iExtension, y);
	}

	glColor3f(0, 0, 0.75f);

	glVertex3i(0, 0, -iExtension);
	glVertex3i(0, 0, iExtension);

	glVertex3i(0, 0, iExtension);
	glVertex3i(0, -iOffset10, iExtension - iOffset5);

	glVertex3i(0, 0, iExtension);
	glVertex3i(0, iOffset10, iExtension - iOffset5);

	glEnd();

	glDepthFunc(GL_ALWAYS);
	glBegin(GL_LINES);

	glColor3f(0, 0.75f, 0);

	glVertex2i(0, iExtension);
	glVertex2i(0, -iExtension);

	glVertex2i(0, iExtension);
	glVertex2i(-iOffset10, iExtension - iOffset5);

	glVertex2i(0, iExtension);
	glVertex2i(iOffset10, iExtension - iOffset5);

	glColor3f(0.75f, 0, 0);

	glVertex2i(iExtension, 0);
	glVertex2i(-iExtension, 0);

	glVertex2i(iExtension, 0);
	glVertex2i(iExtension - iOffset5, -iOffset10);

	glVertex2i(iExtension, 0);
	glVertex2i(iExtension - iOffset5, iOffset10);

	glEnd();

	glColor3f(0, 0, 0);
	glPointSize(3);
	glBegin(GL_POINTS);
	glVertex2i(0, 0);
	glEnd();
	glPointSize(1);

	glDepthFunc(GL_LEQUAL);
}