#define _WIN32_DCOM
#include "stdafx.h"
#include "MODELER.h"

#include <initguid.h>
#include "DCOMserver.h"

#include "MainFrm.h"
#include "IncomingAnimationsBar.h"
#include "Node.h"

#include "Include\anim_format.h"

//==========================================================================================

class CClassFactory : public IClassFactory {
  public:
    // IUnknown
    STDMETHODIMP    QueryInterface (REFIID riid, void** ppv);
    STDMETHODIMP_(ULONG) AddRef(void)  { return 1; };
    STDMETHODIMP_(ULONG) Release(void) { return 1; }

    // IClassFactory
    STDMETHODIMP    CreateInstance (LPUNKNOWN punkOuter, REFIID iid, void **ppv);
    STDMETHODIMP    LockServer (BOOL fLock) { return E_FAIL; };
    };

CClassFactory   g_ClassFactory;

//==========================================================================================

class CModelerObject : public IStream {
  public:
    // IUnknown
    STDMETHODIMP    QueryInterface (REFIID iid, void **ppv);
    STDMETHODIMP_(ULONG) AddRef(void)  { return InterlockedIncrement(&m_cRef); };
    STDMETHODIMP_(ULONG) Release(void) { if (InterlockedDecrement(&m_cRef) == 0) { delete this; return 0; } return 1; }

    // IStream
    STDMETHODIMP    Read(void *pv, ULONG cb, ULONG *pcbRead)
        { return E_FAIL; }
    STDMETHODIMP    Write(VOID const *pv, ULONG cb, ULONG *pcbWritten);
    STDMETHODIMP    Seek(LARGE_INTEGER dbMove, DWORD dwOrigin, ULARGE_INTEGER *pbNewPosition)
        { return E_FAIL; }
    STDMETHODIMP    SetSize(ULARGE_INTEGER cbNewSize)
        { return E_FAIL; }
    STDMETHODIMP    CopyTo(IStream *pstm, ULARGE_INTEGER cb, ULARGE_INTEGER *pcbRead, ULARGE_INTEGER *pcbWritten)
        { return E_FAIL; }
    STDMETHODIMP    Commit(DWORD grfCommitFlags)
        { return E_FAIL; }
    STDMETHODIMP    Revert(void)
        { return E_FAIL; }
    STDMETHODIMP    LockRegion(ULARGE_INTEGER bOffset, ULARGE_INTEGER cb, DWORD dwLockType)
        { return E_FAIL; }
    STDMETHODIMP    UnlockRegion(ULARGE_INTEGER bOffset, ULARGE_INTEGER cb, DWORD dwLockType)
        { return E_FAIL; }
    STDMETHODIMP    Stat(STATSTG *pstatstg, DWORD grfStatFlag)
        { return E_FAIL; }
    STDMETHODIMP    Clone(IStream **ppstm)
        { return E_FAIL; }

    // constructors/destructors
    CModelerObject()     { m_cRef = 1; }
    ~CModelerObject()    { }

  private:
    LONG        m_cRef;
    };

//==========================================================================================

static DWORD   dwRegister;

static BOOL Needed_CoRevokeClassObject=FALSE;
static BOOL Needed_CoUninitialize=FALSE;

BOOL RegisterDCOMServer (char *message)
{
    HRESULT hr;

	*message=0;

	// *** valuta se i servizi DCOM sono disponibili ***

	if ( GetProcAddress(GetModuleHandle("OLE32"),"CoInitializeEx") == NULL ) {
		FormatDCOMMessage("DCOM services not available!",S_OK,message);
		return FALSE;
	}

	// *** valuta se i servizi DCOM sono abilitati ***

	HKEY hKey;
	LONG lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Ole",
		0,
		KEY_ALL_ACCESS,
		&hKey);

	ASSERT(lResult == ERROR_SUCCESS);

	char rgch[2];
	DWORD cb = sizeof(rgch);
	lResult = RegQueryValueEx(hKey,
		TEXT("EnableDCOM"),
		0, NULL, (unsigned char *)rgch, &cb);
	ASSERT(lResult == ERROR_SUCCESS);
	lResult = RegCloseKey(hKey);
	ASSERT(lResult == ERROR_SUCCESS);

	if ( rgch[0] != 'y' && rgch[0] != 'Y' ) {
		FormatDCOMMessage("DCOM services not enabled!",S_OK,message);
		return FALSE;
	}

	// *** registra il server nel registry di windows ***
	{

		CBCGRegistry		reg (FALSE, FALSE);
		reg.m_hKey = HKEY_CLASSES_ROOT;
		if (reg.CreateKey ("CLSID\\{68B07AA7-B13F-4233-85F0-DD7AA844AB32}")) {
			reg.Write ("","MapGen Modeler Server");
			reg.Write ("AppID","{68B07AA7-B13F-4233-85F0-DD7AA844AB32}");
			if (reg.CreateKey ("LocalServer32")) {
				char		filename[1024];
				if ( GetModuleFileName(NULL,filename,sizeof(filename)) != 0 )
					reg.Write ("",filename);
			}
		}

	}
	{

		CBCGRegistry		reg (FALSE, FALSE);
		reg.m_hKey = HKEY_CLASSES_ROOT;
		if (reg.CreateKey ("AppID\\{68B07AA7-B13F-4233-85F0-DD7AA844AB32}")) {
			reg.Write ("","MapGen Modeler Server");
			reg.Write ("RunAs","Interactive User");
		}
	}

	// *** inizializza DCOM ***

    hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (FAILED(hr)) {
		FormatDCOMMessage("CoInitializeEx for free-threading failed!",hr,message);
		return FALSE;
	}
	else
		Needed_CoUninitialize=TRUE;

    // *** register the class-object with OLE ***

    hr = CoRegisterClassObject(CLSID_MapGenModeler, &g_ClassFactory,
        CLSCTX_SERVER, REGCLS_MULTIPLEUSE, &dwRegister);
    if (FAILED(hr)) {
		FormatDCOMMessage("CoRegisterClassObject failed!",hr,message);
		return FALSE;
	}
	else
		Needed_CoRevokeClassObject=TRUE;

	return TRUE;
}

void UnRegisterDCOMServer (void)
{
	// *** CoRevokeClassObject ***

	if ( Needed_CoRevokeClassObject ) CoRevokeClassObject(dwRegister);

	// *** CoUninitialize ***

	if ( Needed_CoUninitialize ) CoUninitialize();
}

void FormatDCOMMessage (LPTSTR szPrefix, HRESULT hr, LPTSTR szOutput)
{
    LPTSTR   szMessage;

    if (hr == S_OK)
        {
        wsprintf(szOutput,TEXT("\n%s\n"),szPrefix);
        return;
        }
 
    if (HRESULT_FACILITY(hr) == FACILITY_WINDOWS)
        hr = HRESULT_CODE(hr);
 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        hr,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), //The user default language
        (LPTSTR)&szMessage,
        0,
        NULL);

    wsprintf(szOutput,TEXT("\n%s\n\n%lx = %s\n"), szPrefix, hr, szMessage);
    
    LocalFree(szMessage);
}

//==========================================================================================

// ---------------------------------------------------------------------------
// %%Function: CModelerObject::QueryInterface
// ---------------------------------------------------------------------------
 STDMETHODIMP
CModelerObject::QueryInterface(REFIID riid, void** ppv)
{
    if (ppv == NULL)
        return E_INVALIDARG;
    if (riid == IID_IUnknown || riid == IID_IStream)
        {
        *ppv = (IUnknown *) this;
        AddRef();
        return S_OK;
        }
    *ppv = NULL;
    return E_NOINTERFACE;
}  // CModelerObject::QueryInterface

// ---------------------------------------------------------------------------
// %%Function: CModelerObject::Write
// ---------------------------------------------------------------------------
 STDMETHODIMP
CModelerObject::Write(VOID const *pv, ULONG cb, ULONG *pcbWritten)
{
    if (!pv || cb < sizeof(anim_stream_desc_t))
        return E_INVALIDARG;
    if (pcbWritten)
        *pcbWritten = cb;

	// *** aggiunge l'animazione alla lista ***

	ASSERT(theApp.MainFrame!=NULL);

	HWND		hwnd=theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimList.m_hWnd;
	ASSERT(hwnd!=NULL);

	anim_stream_desc_t		*stream=(anim_stream_desc_t*)malloc(cb);

	if ( stream ) {

		memcpy(stream,pv,cb);

#if 0
		FILE		*pfImage = fopen("c:\\image.bin","wb");
		ASSERT( pfImage != NULL );
		fwrite(stream, 1, cb, pfImage);
		fclose(pfImage);
#endif

		CSceneRoot	*pSR = new CSceneRoot(stream);

		if ( pSR->m_bCorrupt != FALSE ) {

			delete pSR;

		}
		else {

			EnterCriticalSection
				(&theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimListAccess); // ENTERCRITICALSECTION

			int			index=SendMessage(hwnd,LVM_GETITEMCOUNT,0,0);

			LVITEM		item;

			item.mask=LVIF_TEXT | LVIF_PARAM;
			item.iItem=index;
			item.iSubItem=0;
			item.pszText=strlen(stream->AnimName) ? stream->AnimName : "<<< unnamed >>>";
			item.lParam=(LPARAM)pSR;

			SendMessage(hwnd,LVM_INSERTITEM,0,(LPARAM)&item);

			char		text[1024];
			sprintf(text,"%i",stream->StreamSize);

			item.mask=LVIF_TEXT;
			item.iItem=index;
			item.iSubItem=1;
			item.pszText=text;

			SendMessage(hwnd,LVM_SETITEM,0,(LPARAM)&item);

			item.mask=LVIF_TEXT;
			item.iItem=index;
			item.iSubItem=2;
			item.pszText=stream->ClientName;

			SendMessage(hwnd,LVM_SETITEM,0,(LPARAM)&item);

			LeaveCriticalSection
				(&theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimListAccess); // LEAVECRITICALSECTION

		}

	}

    return S_OK;
}  // CModelerObject::Write

// ---------------------------------------------------------------------------
// %%Function: CClassFactory::QueryInterface
// ---------------------------------------------------------------------------
 STDMETHODIMP
CClassFactory::QueryInterface(REFIID riid, void** ppv)
{
    if (ppv == NULL)
        return E_INVALIDARG;
    if (riid == IID_IClassFactory || riid == IID_IUnknown)
        {
        *ppv = (IClassFactory *) this;
        AddRef();
        return S_OK;
        }
    *ppv = NULL;
    return E_NOINTERFACE;
}  // CClassFactory::QueryInterface

// ---------------------------------------------------------------------------
// %%Function: CClassFactory::CreateInstance
// ---------------------------------------------------------------------------
 STDMETHODIMP
CClassFactory::CreateInstance(LPUNKNOWN punkOuter, REFIID riid, void** ppv)
{
    LPUNKNOWN   punk;
    HRESULT     hr;

    *ppv = NULL;

    if (punkOuter != NULL)
        return CLASS_E_NOAGGREGATION;

    punk = new CModelerObject;

    if (punk == NULL)
        return E_OUTOFMEMORY;

    hr = punk->QueryInterface(riid, ppv);
    punk->Release();
    return hr;
}  // CClassFactory::CreateInstance

//==========================================================================================

void FreeAnimListMemory (void)
{
	ASSERT(theApp.MainFrame!=NULL);

	HWND		hwnd=theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimList.m_hWnd;
	ASSERT(hwnd!=NULL);

	EnterCriticalSection
		(&theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimListAccess); // ENTERCRITICALSECTION

	int			iCount=SendMessage(hwnd,LVM_GETITEMCOUNT,0,0);

	for ( int i=0; i<iCount; i++ ) {

		LVITEM		item;
		item.mask = LVIF_PARAM;
		item.iItem = i;
		item.iSubItem = 0;
		SendMessage(hwnd, LVM_GETITEM, 0, (LPARAM) &item);

		CSceneRoot	*pSR = (CSceneRoot*) item.lParam;

		delete pSR;

	}

	LeaveCriticalSection
		(&theApp.MainFrame->m_wndIncomingAnimationsBar.m_AnimListAccess); // LEAVECRITICALSECTION
}

//==========================================================================================