#include "MapGenProject.h"

#include <assert.h>
#include <initguid.h>

// {223ED916-F6B9-48fa-B440-881FA19783C2}
DEFINE_GUID(CLSID_MapGenProject, 0x223ed916, 0xf6b9, 0x48fa, 0xb4, 0x40, 0x88, 0x1f, 0xa1, 0x97, 0x83, 0xc2);

//------------------------------------------------------------------------------------------

CMapGenProject::CMapGenProject (char *pszResult, void *pInputMem, ULONG ulInputSize, void *pOutputMem, ULONG ulOutputSize, BOOL bLocal /*= TRUE*/, char *pszServer /*= NULL*/)
{
	m_pszResult = pszResult;
	m_pInputMem = pInputMem;
	m_ulInputSize = ulInputSize;
	m_pOutputMem = pOutputMem;
	m_ulOutputSize = ulOutputSize;
	m_bLocal = bLocal;
	m_pszServer = pszServer;

	*m_pszResult = 0;

	// *** crea il thread ***

	DWORD		dwThreadID;
	HANDLE		hThreadH;

	hThreadH=::CreateThread(NULL,
		0,
		fnThread,
		(LPVOID)this,
		0,
		&dwThreadID);

	WaitForSingleObject(hThreadH,INFINITE);
}

//------------------------------------------------------------------------------------------

CMapGenProject::~CMapGenProject ()
{
}

//------------------------------------------------------------------------------------------

void CMapGenProject::Message (LPTSTR szPrefix, HRESULT hr, LPTSTR szOutput)
{
    LPTSTR   szMessage;

    if (hr == S_OK)
        {
        wsprintf(szOutput,TEXT("\n%s\n"),szPrefix);
        return;
        }
 
    if (HRESULT_FACILITY(hr) == FACILITY_WINDOWS)
        hr = HRESULT_CODE(hr);
 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        hr,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), //The user default language
        (LPTSTR)&szMessage,
        0,
        NULL);

    wsprintf(szOutput,TEXT("\n%s\n\n%lx = %s\n"), szPrefix, hr, szMessage);
    
    LocalFree(szMessage);
}

//------------------------------------------------------------------------------------------

DWORD WINAPI CMapGenProject::fnThread (LPVOID pvInstance)
{
	CMapGenProject		*pInstance = (CMapGenProject*)pvInstance;

	// *** valuta se il free threading  disponibile ***

	if ( GetProcAddress(GetModuleHandle("OLE32"),"CoInitializeEx") == NULL ) {
		Message("Free threading not available!",
			S_OK,pInstance->m_pszResult);
		return 1;
	}

	// *** valuta se i servizi DCOM sono abilitati ***

	HKEY hKey;
	LONG lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Ole",
		0,
		KEY_ALL_ACCESS,
		&hKey);

	assert(lResult == ERROR_SUCCESS);

	char rgch[2];
	DWORD cb = sizeof(rgch);
	lResult = RegQueryValueEx(hKey,
		TEXT("EnableDCOM"),
		0, NULL, (unsigned char *)rgch, &cb);
	assert(lResult == ERROR_SUCCESS);
	lResult = RegCloseKey(hKey);
	assert(lResult == ERROR_SUCCESS);

	if ( rgch[0] != 'y' && rgch[0] != 'Y' ) {
		Message("DCOM services not enabled!",
			S_OK,pInstance->m_pszResult);
		return 1;
	}

	// *** CoInitializeEx ***

	HRESULT hr;
    MULTI_QI    mq;
    COSERVERINFO csi, *pcsi=NULL;
    WCHAR wsz [MAX_PATH];

    hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (FAILED(hr)) {
		Message("CoInitializeEx for free-threading failed!",
			hr,pInstance->m_pszResult);
		return 1;
	}

	// *** crea una istanza locale o remota ***

	if ( pInstance->m_bLocal==FALSE && pInstance->m_pszServer!=NULL ) {
        MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, pInstance->m_pszServer, -1,
            wsz, MAX_PATH);
        memset(&csi, 0, sizeof(COSERVERINFO));
        csi.pwszName = wsz;
        pcsi = &csi;
	}

    mq.pIID = &IID_IStream;
    mq.pItf = NULL;
    mq.hr = S_OK;
    hr = CoCreateInstanceEx(CLSID_MapGenProject, NULL, CLSCTX_SERVER, pcsi, 1, &mq);
    if (FAILED(hr)) {
		Message("CoCreateInstanceEx of CLSID_MapGenProject failed!",
			hr,pInstance->m_pszResult);
		CoUninitialize();
		return 1;
	}

	// *** invia i dati ***

	LPSTREAM    pstm = (IStream*)mq.pItf;

	hr = pstm->Write(pInstance->m_pInputMem, pInstance->m_ulInputSize, NULL);

    if (FAILED(hr)) {
		Message("Unable to send data to the server!",
			hr,pInstance->m_pszResult);
		pstm->Release();
		CoUninitialize();
		return 1;
	}

	hr = pstm->Read(pInstance->m_pOutputMem, pInstance->m_ulOutputSize, NULL);

    if (FAILED(hr)) {
		Message("Unable to receive data from the server!",
			hr,pInstance->m_pszResult);
		pstm->Release();
		CoUninitialize();
		return 1;
	}

	pstm->Release();

	// *** CoUninitialize ***

	CoUninitialize();

	// ritorna
	return 0;
}

//------------------------------------------------------------------------------------------