#include "Node.h"

#include <gl/gl.h>		// OpenGL stuff
#include <gl/glu.h>
#include <gl/glaux.h>

//==========================================================================================

CSceneRoot::CSceneRoot (anim_stream_desc_t *pAnim) : CNode(NULL), m_pSML(NULL), m_bCorrupt(FALSE), m_iNumOfNodes(1)
{
	m_eType = eSceneRoot;
	m_strName = "Scene Root";
	m_iIndex = 0;
	m_mId2Obj[0] = this;

	SMaterial		mVoidMat;
	mVoidMat.m_iIndex = 0;
	mVoidMat.m_strName = "null";
	mVoidMat.m_iTris = 0;
	m_vMaterials.push_back(mVoidMat);

	BYTE *pStart = (BYTE*)pAnim;
	m_pAnimPos = pStart;

	m_pASD = pAnim;
	m_pAnimPos += sizeof(anim_stream_desc_t);

	m_pGI = (global_info_t*)m_pAnimPos;
	m_pAnimPos += sizeof(global_info_t);

	// attraversa lo stream

	while ( m_pAnimPos - pStart < pAnim->StreamSize ) {

		tag_type	*pTag = (tag_type*)m_pAnimPos;

		switch ( *pTag ) {
		case TAG_GROUP:
			{
				scene_group_t	*pSG = (scene_group_t*)m_pAnimPos;
				map<int, CNode *>::iterator it = m_mId2Obj.find(pSG->parent_node_id);
				if ( it == m_mId2Obj.end() )
					m_bCorrupt = TRUE;
				else
					new CGroup(it->second, pSG);
			}
			break;
		case TAG_GEOMOBJ:
			{
				scene_geomobj_t		*pSGO = (scene_geomobj_t*)m_pAnimPos;
				map<int, CNode *>::iterator it = m_mId2Obj.find(pSGO->parent_node_id);
				if ( it == m_mId2Obj.end() )
					m_bCorrupt = TRUE;
				else
					new CGeomObj(it->second, pSGO);
			}
			break;
		case TAG_MATLIST:
			{
				m_pSML = (scene_matlist_t*)m_pAnimPos;
				m_pAnimPos += sizeof(scene_matlist_t);

				for ( int i=0; i<m_pSML->mat_count; i++ ) {

					SMaterial		mMat;

					scene_material_t	*pSM = (scene_material_t*)m_pAnimPos;
					m_pAnimPos += sizeof(scene_material_t);

					char		szName[1024];
					memcpy(szName, m_pAnimPos, pSM->name_lenght);
					szName[pSM->name_lenght] = 0;
					m_pAnimPos += pSM->name_lenght;

					mMat.m_iIndex = pSM->material_id;
					mMat.m_strName = szName;
					mMat.m_iTris = 0;

					m_vMaterials.push_back(mMat);

				}
			}
			break;
		default:
			m_bCorrupt = TRUE;
			break;
		}

		if ( m_bCorrupt != FALSE )
			return;

	}

	// chiama PostProcess

	try {
		PostProcess();
	}
	catch (...)
	{
		m_bCorrupt = TRUE;
	}
}

//==========================================================================================

CGroup::CGroup (CNode *pParent, scene_group_t *pGroupHead) : CNode(pParent)
{
	CSceneRoot			*pSR = (CSceneRoot*)m_pSceneRoot;

	pSR->m_iNumOfNodes++;

	m_pSG = (scene_group_t*)pSR->m_pAnimPos;
	pSR->m_pAnimPos += sizeof(scene_group_t);

	char		szName[1024];
	memcpy(szName, pSR->m_pAnimPos, m_pSG->node_name_lenght);
	szName[m_pSG->node_name_lenght] = 0;
	pSR->m_pAnimPos += m_pSG->node_name_lenght;

	m_eType = eGroup;
	m_strName = szName;
	m_iIndex = m_pSG->node_id;
	pSR->m_mId2Obj[m_iIndex] = this;
}

//==========================================================================================

CGeomObj::CGeomObj (CNode *pParent, scene_geomobj_t *pGeomObj) : CNode(pParent),
	m_bInterpolableMesh(FALSE),
	m_pInterpolatedMeshVerts(NULL),
	m_pInterpolatedMeshNormals(NULL),
	m_pDeformationMesh(NULL),
	m_bInterpolateNormals(FALSE),
	m_pRenderableVerts(NULL)
{
	Keys.m_pSK = NULL;
	Keys.m_pTCBpk = NULL;
	Keys.m_pTCBrk = NULL;
	Keys.m_pTCBsk = NULL,

	Samples.m_pSS = NULL;
	Samples.m_pTM = NULL;

	m_pPhysVerts = NULL;

	CSceneRoot			*pSR = (CSceneRoot*)m_pSceneRoot;

	pSR->m_iNumOfNodes++;

	m_pSGO = (scene_geomobj_t*)pSR->m_pAnimPos;
	pSR->m_pAnimPos += sizeof(scene_geomobj_t);

	m_mNodeTM = IdentityMatrix;
	MatrixFromAnimMatrix3(&m_pSGO->figure_tm, &m_mFigureTM);

	char		szName[1024];
	memcpy(szName, pSR->m_pAnimPos, m_pSGO->node_name_lenght);
	szName[m_pSGO->node_name_lenght] = 0;
	pSR->m_pAnimPos += m_pSGO->node_name_lenght;

	m_eType = eGeomObj;
	m_strName = szName;
	m_iIndex = m_pSGO->node_id;
	pSR->m_mId2Obj[m_iIndex] = this;

	BYTE *pStart = (BYTE*)pSR->m_pASD;

	// attraversa lo stream

	while ( pSR->m_pAnimPos - pStart < pSR->m_pASD->StreamSize ) {

		tag_type	*pTag = (tag_type*)pSR->m_pAnimPos;

		switch ( *pTag ) {
		case TAG_KEYS:
			{
				Keys.m_pSK = (scene_keys_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(scene_keys_t);

				// position keys

				Keys.m_pTCBpk = (TCB_POSkey_t*)pSR->m_pAnimPos;

				switch ( Keys.m_pSK->position_controller ) {
				case POSITIONCONTROLLER_TCB:
					pSR->m_pAnimPos += sizeof(TCB_POSkey_t) * Keys.m_pSK->position_keys;
					break;
				case POSITIONCONTROLLER_HYBRID:
					pSR->m_pAnimPos += sizeof(HYB_POSkey_t) * Keys.m_pSK->position_keys;
					break;
				case POSITIONCONTROLLER_LIN:
					pSR->m_pAnimPos += sizeof(LIN_POSkey_t) * Keys.m_pSK->position_keys;
					break;
				}

				// rotation keys

				Keys.m_pTCBrk = (TCB_ROTkey_t*)pSR->m_pAnimPos;

				switch ( Keys.m_pSK->rotation_controller ) {
				case ROTATIONCONTROLLER_TCB:
					pSR->m_pAnimPos += sizeof(TCB_ROTkey_t) * Keys.m_pSK->rotation_keys;
					break;
				case ROTATIONCONTROLLER_HYBRID:
					pSR->m_pAnimPos += sizeof(HYB_ROTkey_t) * Keys.m_pSK->rotation_keys;
					break;
				case ROTATIONCONTROLLER_LIN:
					pSR->m_pAnimPos += sizeof(LIN_ROTkey_t) * Keys.m_pSK->rotation_keys;
					break;
				}

				// scale keys

				Keys.m_pTCBsk = (TCB_SCLkey_t*)pSR->m_pAnimPos;

				switch ( Keys.m_pSK->scale_controller ) {
				case SCALECONTROLLER_TCB:
					pSR->m_pAnimPos += sizeof(TCB_SCLkey_t) * Keys.m_pSK->scale_keys;
					break;
				case SCALECONTROLLER_HYBRID:
					pSR->m_pAnimPos += sizeof(HYB_SCLkey_t) * Keys.m_pSK->scale_keys;
					break;
				case SCALECONTROLLER_LIN:
					pSR->m_pAnimPos += sizeof(LIN_SCLkey_t) * Keys.m_pSK->scale_keys;
					break;
				}

			}
			break;
		case TAG_SAMPTM:
			{
				Samples.m_pSS = (scene_samptm_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(scene_samptm_t);

				Samples.m_pTM = (TMSample_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(TMSample_t) * Samples.m_pSS->keys_num;
			}
			break;
		case TAG_MESH:
			{
				SMesh		mMesh;

				mMesh.m_pSM = (scene_mesh_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(scene_mesh_t);

				mMesh.m_pVerts = (AnimPoint3_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(AnimPoint3_t) * mMesh.m_pSM->num_verts;

				mMesh.m_pFaces = (AnimFace_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(AnimFace_t) * mMesh.m_pSM->num_faces;

				if ( mMesh.m_pSM->has_normals == 0 ) {
					mMesh.m_pNormals = NULL;
				}
				else {
					mMesh.m_pNormals = (AnimFaceNormal_t*)pSR->m_pAnimPos;
					pSR->m_pAnimPos += sizeof(AnimFaceNormal_t) * mMesh.m_pSM->num_faces;
				}

				if ( mMesh.m_pSM->num_tverts == 0 ) {
					mMesh.m_pTVerts = NULL;
					mMesh.m_pTFaces = NULL;
				}
				else {

					mMesh.m_pTVerts = (AnimUVVert_t*)pSR->m_pAnimPos;
					pSR->m_pAnimPos += sizeof(AnimUVVert_t) * mMesh.m_pSM->num_tverts;

					mMesh.m_pTFaces = (AnimTVFace_t*)pSR->m_pAnimPos;
					pSR->m_pAnimPos += sizeof(AnimTVFace_t) * mMesh.m_pSM->num_faces;

				}

				m_vMeshes.push_back(mMesh);

				if ( m_vMeshes.size() == 1 )
				{
					MatrixFromAnimMatrix3(&mMesh.m_pSM->tm, &m_mNodeTM);
				}
			}
			break;
		case TAG_PHYSVERTS:
			{
				m_pPhysVerts = (scene_physverts_t*)pSR->m_pAnimPos;
				pSR->m_pAnimPos += sizeof(scene_physverts_t);

				for ( int i=0; i<m_pPhysVerts->num_verts; i++ ) {

					SPhysVert		vVert;

					tag_type	*pTag = (tag_type*)pSR->m_pAnimPos;

					switch ( *pTag ) {
					case TAG_RIGIDVERT:
						{
							vVert.m_pSRV = (scene_rigidvert_t*)pSR->m_pAnimPos;
							pSR->m_pAnimPos += sizeof(scene_rigidvert_t);

							vVert.m_vBones.push_back((CNode*)vVert.m_pSRV->bone_id);
							vVert.m_vWeights.push_back(1);
						}
						break;
					case TAG_BLENDEDVERT:
						{
							vVert.m_pSBV = (scene_blendedvert_t*)pSR->m_pAnimPos;
							pSR->m_pAnimPos += sizeof(scene_blendedvert_t);

							for ( int j=0; j<vVert.m_pSBV->bone_num; j++ ) {

								scene_boneweight_t	*pSBW = (scene_boneweight_t*)pSR->m_pAnimPos;
								pSR->m_pAnimPos += sizeof(scene_boneweight_t);

								vVert.m_vBones.push_back((CNode*)pSBW->bone_id);
								vVert.m_vWeights.push_back(pSBW->bone_weight);

							}
						}
						break;
					default:
						pSR->m_bCorrupt = TRUE;
						return;
					}

					m_vPhysVerts.push_back(vVert);

				}
			}
			break;
		default:
			return;
		}

	}
}

//==========================================================================================

void CGeomObj::PostProcess ()
{
	CSceneRoot	*pSR = (CSceneRoot*)m_pSceneRoot;

	// si occupa di m_vBones

	int		i,j;

	for ( i=0; i<m_vPhysVerts.size(); i++ ) {

		CGeomObj::SPhysVert	*pPV = &m_vPhysVerts[i];

		for ( j=0; j<pPV->m_vBones.size(); j++ ) {

			map<int, CNode *>::iterator it = pSR->m_mId2Obj.find((int)pPV->m_vBones[j]);

			if ( it == pSR->m_mId2Obj.end() )
				pSR->m_bCorrupt = TRUE;
			else
				pPV->m_vBones[j] = it->second;

		}

	}

	// si occupa m_vFaceMaterials

	for ( i=0; i<m_vMeshes.size(); i++ ) {

		CGeomObj::SMesh		*pM = &m_vMeshes[i];

		for ( j=0; j<pM->m_pSM->num_faces; j++ ) {

			AnimFace_t		*pAF = &pM->m_pFaces[j];

			for ( int k=0; k<pSR->m_vMaterials.size(); k++ )
				if ( pSR->m_vMaterials[k].m_iIndex == pAF->material_id )
				{
					pSR->m_vMaterials[k].m_iTris ++;
					break;
				}

			if ( k == pSR->m_vMaterials.size() )
				pSR->m_bCorrupt = TRUE;
			else
				pM->m_vFaceMaterials.push_back(k);

		}

	}

	// si occupa di m_bInterpolableMesh

	if ( m_pSGO->node_type == NODETYPE_DEFORMATION &&
		m_vMeshes.size() > 1 ) {

			m_bInterpolableMesh = TRUE;

			SMesh	*pFirst = &m_vMeshes[0];

			for ( i=1; i<m_vMeshes.size(); i++ ) {

				SMesh	*pMesh = &m_vMeshes[i];

				// effettua un controllo sul numero di vertici e facce

				if ( pFirst->m_pSM->num_verts != pMesh->m_pSM->num_verts ||
					pFirst->m_pSM->num_faces != pMesh->m_pSM->num_faces ) {

						m_bInterpolableMesh = FALSE;
						break;

				}

				// controlla che le facce siano identiche

				for ( j=0; j<pMesh->m_pSM->num_faces; j++ ) {

					AnimFace_t		*pFace0 = &pFirst->m_pFaces[j];
					AnimFace_t		*pFace1 = &pMesh->m_pFaces[j];

					if ( pFace0->v[0] != pFace1->v[0] ||
						pFace0->v[1] != pFace1->v[1] ||
						pFace0->v[2] != pFace1->v[2] )
							break;

				}

				if ( j != pMesh->m_pSM->num_faces ) {
					m_bInterpolableMesh = FALSE;
					break;
				}

			}

			if ( m_bInterpolableMesh != FALSE ) {

				m_pInterpolatedMeshVerts = (AnimPoint3_t*)malloc( sizeof(AnimPoint3_t) * m_vMeshes[0].m_pSM->num_verts );
				if ( m_pInterpolatedMeshVerts == NULL )
					throw;

				if ( m_bInterpolateNormals != FALSE ) {

					m_pInterpolatedMeshNormals = (AnimFaceNormal_t*)malloc( sizeof(AnimFaceNormal_t) * m_vMeshes[0].m_pSM->num_faces );
					if ( m_pInterpolatedMeshNormals == NULL )
						throw;

				}

			}

	}

	// controlla le informazioni sul physique modifier

	if ( m_pSGO->node_type == NODETYPE_PHYSIQUE ) {

		if ( m_pPhysVerts == NULL || m_vMeshes.size() != 1 ) {

			pSR->m_bCorrupt = TRUE;

		}
		else if ( m_pPhysVerts->num_verts != m_vMeshes[0].m_pSM->num_verts ) {

			pSR->m_bCorrupt = TRUE;

		}

	}

	// alloca m_pRenderableVerts

	int		iNumVerts = 0;

	for ( i=0; i<m_vMeshes.size(); i++ )
		if ( m_vMeshes[i].m_pSM->num_verts > iNumVerts )
			iNumVerts = m_vMeshes[i].m_pSM->num_verts;

	if ( iNumVerts ) {
		m_pRenderableVerts = (AnimPoint3_t*)malloc( sizeof(AnimPoint3_t) * iNumVerts );
		if ( m_pRenderableVerts == NULL )
			throw;
	}

	// controlla che il rigid node abbia una sola mesh

	if ( m_pSGO->node_type == NODETYPE_RIGID &&
		m_vMeshes.size() != 1 ) {

			pSR->m_bCorrupt = TRUE;

	}

	// controlla che un deformation node abbia almeno due mesh

	if ( m_pSGO->node_type == NODETYPE_DEFORMATION &&
		m_vMeshes.size() <= 1 ) {

			pSR->m_bCorrupt = TRUE;

	}

	// processa tutti i children
	CNode::PostProcess();
}

//==========================================================================================

void CGeomObj::EvalWorldState (TimeValue time)
{
	CSceneRoot	*pSR = (CSceneRoot*)m_pSceneRoot;

	// valuta se interpolare la mesh

	if ( m_pSGO->node_type == NODETYPE_DEFORMATION ) {

		// valuta se la mesh  interpolabile

		if ( m_bInterpolableMesh != FALSE ) {

			SMesh		*pFirst;
			SMesh		*pSecond;

			int			i;

			// stabilisce le due mesh tra cui interpolare

			for ( i=1; i<m_vMeshes.size(); i++ ) {

				pFirst = &m_vMeshes[i-1];
				pSecond = &m_vMeshes[i];

				if ( time >= pFirst->m_pSM->time &&
					time <= pSecond->m_pSM->time )
						break;

			}

			// interpola le mesh

			if ( time == pFirst->m_pSM->time ) {

				m_pDeformationMesh = pFirst;

			}
			else if ( time == pSecond->m_pSM->time ) {

				m_pDeformationMesh = pSecond;

			}
			else {

				m_pDeformationMesh = NULL;

				float		fI = ( time - pFirst->m_pSM->time ) / ( (float)pSecond->m_pSM->time - pFirst->m_pSM->time );

				int			iNumVerts = pFirst->m_pSM->num_verts;
				int			iNumFaces = pFirst->m_pSM->num_faces;

				// interpola i vertici della mesh

				for ( i=0; i<iNumVerts; i++ )
					Interpolate(&pFirst->m_pVerts[i], &pSecond->m_pVerts[i],
						fI, &m_pInterpolatedMeshVerts[i]);

				// interpola le normali

				if ( m_bInterpolateNormals != FALSE && pFirst->m_pNormals != NULL && pSecond->m_pNormals != NULL )
					for ( i=0; i<iNumFaces; i++ ) {

						Interpolate(&pFirst->m_pNormals[i].face_normal,
							&pSecond->m_pNormals[i].face_normal,
							fI, &m_pInterpolatedMeshNormals[i].face_normal);
						VectorNormalize(&m_pInterpolatedMeshNormals[i].face_normal);

						for ( int j=0; j<3; j++ ) {

							Interpolate(&pFirst->m_pNormals[i].vertex_normals[j],
								&pSecond->m_pNormals[i].vertex_normals[j],
								fI, &m_pInterpolatedMeshNormals[i].vertex_normals[j]);
							VectorNormalize(&m_pInterpolatedMeshNormals[i].vertex_normals[j]);

						}

					}

			}

		}
		else {

			// stabilisce la mesh che rappresenta l'oggetto al tempo time

			m_pDeformationMesh = &m_vMeshes[0];

			for ( int i=1; i<m_vMeshes.size(); i++ ) {

				SMesh	*pMesh = &m_vMeshes[i];

				if ( pMesh->m_pSM->time > time ) {

					TimeValue		tAverage = ( pMesh->m_pSM->time - m_vMeshes[i-1].m_pSM->time ) / 2;

					if ( tAverage < time )
						m_pDeformationMesh = pMesh;

					break;

				}
				else
					m_pDeformationMesh = pMesh;

			}

		}

	}

	// interpola linearmente tra i keys o i sample per calcolare la tm

	if ( m_pSGO->node_type == NODETYPE_BIPEDBONE ||
		m_pSGO->node_type == NODETYPE_RIGID ||
		m_pSGO->node_type == NODETYPE_DEFORMATION ) {

			// interpola linearmente la posizione

			vector_t		vPos = m_pSGO->affine_tm.t;

			if ( Keys.m_pSK != NULL ) {
				if ( Keys.m_pSK->position_keys != 0 ) {

					switch ( Keys.m_pSK->position_controller ) {
					case POSITIONCONTROLLER_TCB:
						{

							int		iNumKeys = Keys.m_pSK->position_keys;

							TimeValue	tT[2];
							vector_t	vV[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pTCBpk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pTCBpk[i-1].time;
								vV[0] = Keys.m_pTCBpk[i-1].val;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								vV[0] = Keys.m_pTCBpk[0].val;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pTCBpk[i].time;
								vV[1] = Keys.m_pTCBpk[i].val;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								vV[1] = Keys.m_pTCBpk[iNumKeys-1].val;
							}

							if ( time == tT[0] )
							{
								vPos = vV[0];
							}
							else if ( time == tT[1] )
							{
								vPos = vV[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								Interpolate(&vV[0], &vV[1], fI, &vPos);

							}

						}
						break;
					case POSITIONCONTROLLER_HYBRID:
						{

							int		iNumKeys = Keys.m_pSK->position_keys;

							TimeValue	tT[2];
							vector_t	vV[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pHYBpk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pHYBpk[i-1].time;
								vV[0] = Keys.m_pHYBpk[i-1].val;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								vV[0] = Keys.m_pHYBpk[0].val;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pHYBpk[i].time;
								vV[1] = Keys.m_pHYBpk[i].val;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								vV[1] = Keys.m_pHYBpk[iNumKeys-1].val;
							}

							if ( time == tT[0] )
							{
								vPos = vV[0];
							}
							else if ( time == tT[1] )
							{
								vPos = vV[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								Interpolate(&vV[0], &vV[1], fI, &vPos);

							}

						}
						break;
					case POSITIONCONTROLLER_LIN:
						{

							int		iNumKeys = Keys.m_pSK->position_keys;

							TimeValue	tT[2];
							vector_t	vV[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pLINpk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pLINpk[i-1].time;
								vV[0] = Keys.m_pLINpk[i-1].val;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								vV[0] = Keys.m_pLINpk[0].val;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pLINpk[i].time;
								vV[1] = Keys.m_pLINpk[i].val;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								vV[1] = Keys.m_pLINpk[iNumKeys-1].val;
							}

							if ( time == tT[0] )
							{
								vPos = vV[0];
							}
							else if ( time == tT[1] )
							{
								vPos = vV[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								Interpolate(&vV[0], &vV[1], fI, &vPos);

							}

						}
						break;
					}

				}
			}
			else if ( Samples.m_pSS != NULL ) {

				int		iNumKeys = Samples.m_pSS->keys_num;

				TimeValue	tT[2];
				vector_t	vV[2];

				for ( int i=1; i<iNumKeys; i++ ) {

					TMSample_t		*pFirst = &Samples.m_pTM[i-1];
					TMSample_t		*pSecond = &Samples.m_pTM[i];

					if ( time >= pFirst->time &&
						time <= pSecond->time ) {

							tT[0] = pFirst->time;
							tT[1] = pSecond->time;

							vV[0] = pFirst->tm.t;
							vV[1] = pSecond->tm.t;

							break;

					}

				}

				if ( i<iNumKeys ) {

					if ( time == tT[0] )
					{
						vPos = vV[0];
					}
					else if ( time == tT[1] )
					{
						vPos = vV[1];
					}
					else
					{

						float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

						Interpolate(&vV[0], &vV[1], fI, &vPos);

					}

				}

			}

			// slerpa i quaternioni della rotazione

			AnimQuat_t	qRot = m_pSGO->affine_tm.q;

			if ( Keys.m_pSK != NULL ) {
				if ( Keys.m_pSK->rotation_keys != 0 ) {

					switch ( Keys.m_pSK->rotation_controller ) {
					case ROTATIONCONTROLLER_TCB:
						{

							int		iNumKeys = Keys.m_pSK->rotation_keys;

							TimeValue		tT[2];
							AnimQuat_t		qQ[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pTCBrk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pTCBrk[i-1].time;
								qQ[0] = Keys.m_pTCBrk[i-1].key;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								qQ[0] = Keys.m_pTCBrk[0].key;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pTCBrk[i].time;
								qQ[1] = Keys.m_pTCBrk[i].key;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								qQ[1] = Keys.m_pTCBrk[iNumKeys-1].key;
							}

							if ( time == tT[0] )
							{
								qRot = qQ[0];
							}
							else if ( time == tT[1] )
							{
								qRot = qQ[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								QuatSlerp(&qQ[0], &qQ[1], fI, &qRot);

							}

						}
						break;
					case ROTATIONCONTROLLER_HYBRID:
						{

							int		iNumKeys = Keys.m_pSK->rotation_keys;

							TimeValue		tT[2];
							AnimQuat_t		qQ[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pHYBrk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pHYBrk[i-1].time;
								qQ[0] = Keys.m_pHYBrk[i-1].key;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								qQ[0] = Keys.m_pHYBrk[0].key;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pHYBrk[i].time;
								qQ[1] = Keys.m_pHYBrk[i].key;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								qQ[1] = Keys.m_pHYBrk[iNumKeys-1].key;
							}

							if ( time == tT[0] )
							{
								qRot = qQ[0];
							}
							else if ( time == tT[1] )
							{
								qRot = qQ[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								QuatSlerp(&qQ[0], &qQ[1], fI, &qRot);

							}

						}
						break;
					case ROTATIONCONTROLLER_LIN:
						{

							int		iNumKeys = Keys.m_pSK->rotation_keys;

							TimeValue		tT[2];
							AnimQuat_t		qQ[2];

							for ( int i=0; i<iNumKeys; i++ )
								if ( Keys.m_pLINrk[i].time > time )
									break;

							if ( i!=0 ) {
								tT[0] = Keys.m_pLINrk[i-1].time;
								qQ[0] = Keys.m_pLINrk[i-1].key;
							}
							else {
								tT[0] = pSR->m_pGI->first_frame * pSR->m_pGI->ticks_per_frame;
								qQ[0] = Keys.m_pLINrk[0].key;
							}

							if ( i!=iNumKeys ) {
								tT[1] = Keys.m_pLINrk[i].time;
								qQ[1] = Keys.m_pLINrk[i].key;
							}
							else {
								tT[1] = pSR->m_pGI->last_frame * pSR->m_pGI->ticks_per_frame;
								qQ[1] = Keys.m_pLINrk[iNumKeys-1].key;
							}

							if ( time == tT[0] )
							{
								qRot = qQ[0];
							}
							else if ( time == tT[1] )
							{
								qRot = qQ[1];
							}
							else
							{

								float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

								QuatSlerp(&qQ[0], &qQ[1], fI, &qRot);

							}

						}
						break;
					}

				}
			}
			else if ( Samples.m_pSS != NULL ) {

				int		iNumKeys = Samples.m_pSS->keys_num;

				TimeValue	tT[2];
				AnimQuat_t	qQ[2];

				for ( int i=1; i<iNumKeys; i++ ) {

					TMSample_t		*pFirst = &Samples.m_pTM[i-1];
					TMSample_t		*pSecond = &Samples.m_pTM[i];

					if ( time >= pFirst->time &&
						time <= pSecond->time ) {

							tT[0] = pFirst->time;
							tT[1] = pSecond->time;

							qQ[0] = pFirst->tm.q;
							qQ[1] = pSecond->tm.q;

							break;

					}

				}

				if ( i<iNumKeys ) {

					if ( time == tT[0] )
					{
						qRot = qQ[0];
					}
					else if ( time == tT[1] )
					{
						qRot = qQ[1];
					}
					else
					{

						float	fI = ( time - tT[0] ) / ( (float)tT[1] - tT[0] );

						QuatSlerp(&qQ[0], &qQ[1], fI, &qRot);

					}

				}

			}

			// crea la tm

			matrix_t		mR;
			matrix_t		mT;

			QuatToMatrix(&qRot, &mR);
			MatrixGenerateTranslation(&mT, &vPos);

			MatrixMultiply(&mR, &mT, &m_mTM);

			CNode	*pGeomParent = this;
			BOOL	bFound = FALSE;

			while ( pGeomParent->m_pParent ) {
				pGeomParent = pGeomParent->m_pParent;
				if ( pGeomParent->m_eType == eGeomObj ) {
					bFound = TRUE;
					break;
				}
			}

			if ( bFound != FALSE ) {

				MatrixMultiply(&m_mTM, &((CGeomObj*)pGeomParent)->m_mTM, &m_mTM);

			}

			m_mFinalTM = m_mTM;

			if ( m_pSGO->node_type == NODETYPE_RIGID ||
				m_pSGO->node_type == NODETYPE_DEFORMATION )
					MatrixMultiply(&m_mNodeTM, &m_mFinalTM, &m_mFinalTM);
			else if ( m_pSGO->node_type == NODETYPE_BIPEDBONE )
					MatrixMultiply(&m_mFigureTM, &m_mFinalTM, &m_mFinalTM);

	}

	CNode::EvalWorldState(time);
}

void CGeomObj::GenerateRenderableVerts ()
{
	if ( m_pRenderableVerts == NULL )
		return;

	unsigned short		usType = m_pSGO->node_type;

	if ( usType == NODETYPE_RIGID || ( usType == NODETYPE_DEFORMATION && m_pDeformationMesh )) {

		SMesh	*pMesh = usType == NODETYPE_RIGID ? &m_vMeshes[0] : m_pDeformationMesh;

		int		iNumVerts = pMesh->m_pSM->num_verts;

		for ( int i=0; i<iNumVerts; i++ )
			MatrixApply(&m_mFinalTM, &pMesh->m_pVerts[i], &m_pRenderableVerts[i]);

	}
	else if ( usType == NODETYPE_DEFORMATION ) { // m_pDeformationMesh==NULL

		int		iNumVerts = m_vMeshes[0].m_pSM->num_verts;

		for ( int i=0; i<iNumVerts; i++ )
			MatrixApply(&m_mFinalTM, &m_pInterpolatedMeshVerts[i], &m_pRenderableVerts[i]);

	}
	else if ( usType == NODETYPE_PHYSIQUE ) {

		int		iNumVerts = m_vMeshes[0].m_pSM->num_verts;

		for ( int i=0; i<iNumVerts; i++ ) {

			AnimPoint3_t	*pV = &m_vMeshes[0].m_pVerts[i];
			AnimPoint3_t	*pO = &m_pRenderableVerts[i];
			SPhysVert		*pP = &m_vPhysVerts[i];

			if ( pP->m_vBones.size() == 1 ) { // rigid

				MatrixApply( &((CGeomObj*)pP->m_vBones[0])->m_mFinalTM, pV, pO );

			}
			else { // blended

				memset(pO, 0, sizeof(AnimPoint3_t));

				int		iNumBones = pP->m_vBones.size();

				for ( int j=0; j<iNumBones; j++ ) {

					CGeomObj	*pBone = (CGeomObj*)pP->m_vBones[j];
					float		fWeight = pP->m_vWeights[j];

					vertex_t	vV;

					MatrixApply( &pBone->m_mFinalTM, pV, &vV );
					VectorMultByScalar( &vV, fWeight, &vV );
					VectorAdd( pO, &vV, pO );

				}

			}

		}

	}
}

void CGeomObj::Render (ERenderMode eMode)
{
	// genera i vertici dei triangoli da renderizzare

	if ( m_pRenderableVerts != NULL ) {

		GenerateRenderableVerts();

		// renderizza

		unsigned short		usType = m_pSGO->node_type;

		if ( usType == NODETYPE_RIGID ||
			usType == NODETYPE_DEFORMATION ||
			usType == NODETYPE_PHYSIQUE ) {

				if ( eMode == eWireframeSpecialGreen )
				{
					glColor3f(0,1,0);
				}
				else if ( eMode == eWireframeSpecialRed )
				{
					glColor3f(1,0,0);
				}

				SMesh	*pMesh = (usType == NODETYPE_DEFORMATION && m_pDeformationMesh) ? m_pDeformationMesh : &m_vMeshes[0];

				int		iNumFaces = pMesh->m_pSM->num_faces;

				glBegin(GL_TRIANGLES);
				for ( int i=0; i<iNumFaces; i++ ) {

					AnimFace_t		*pFace = &pMesh->m_pFaces[i];

					if ( eMode == eSolid )
					{
						vector_t		vecV[2], vecNorm;

						VectorSubtract( &m_pRenderableVerts[pFace->v[1]], &m_pRenderableVerts[pFace->v[0]], &vecV[0] );
						VectorSubtract( &m_pRenderableVerts[pFace->v[2]], &m_pRenderableVerts[pFace->v[0]], &vecV[1] );

						VectorCrossP( &vecV[0], &vecV[1], &vecNorm );

						glNormal3fv((float*)&vecNorm);
					}

					for ( int j=0; j<3; j++ )
						glVertex3fv((float*)&m_pRenderableVerts[pFace->v[j]]);

				}
				glEnd();

		}

	}

	CNode::Render(eMode == eWireframeSpecialGreen ? eWireframeSpecialRed : eMode);
}

//==========================================================================================

void CSceneRoot::SetTreeCtrlWithInfo (HWND hTreeCtrl)
{
	// *** cancella tutti gli elementi del tree control ***

	::SendMessage(hTreeCtrl, TVM_DELETEITEM, 0, (LPARAM)TVI_ROOT);

	// *** inserisce alcune informazioni iniziali ***

	char		szString[1024];

	sprintf(szString, "Animation Name: \"%s\"", m_pASD->AnimName);

	InsertItem(hTreeCtrl, 7, szString);

	sprintf(szString, "Raw Animation Size: %i bytes", m_pASD->StreamSize);

	InsertItem(hTreeCtrl, 0, szString);

	sprintf(szString, "Number Of Nodes: %i node(s)", m_iNumOfNodes);

	InsertItem(hTreeCtrl, 0, szString);

	sprintf(szString, "First Frame: %i", m_pGI->first_frame);

	InsertItem(hTreeCtrl, 0, szString);

	sprintf(szString, "Last Frame: %i", m_pGI->last_frame);

	InsertItem(hTreeCtrl, 0, szString);

	sprintf(szString, "Frame Speed: %i fps", m_pGI->frame_speed);

	InsertItem(hTreeCtrl, 7, szString);

	sprintf(szString, "Ticks Per Frame: %i ticks", m_pGI->ticks_per_frame);

	InsertItem(hTreeCtrl, 0, szString);

	// *** aggiunge la lista di materiali ***

	sprintf(szString, "Materials (%i)", m_vMaterials.size());

	HTREEITEM	hMaterialsItem = InsertItem(hTreeCtrl, 1, szString);

	int			i;

	for ( i=0; i<m_vMaterials.size(); i++ )
	{
		// *** aggiunge il nome del materiale ***

		HTREEITEM	hMaterialItem = InsertItem(hTreeCtrl, 2, (LPTSTR)m_vMaterials[i].m_strName.c_str(), hMaterialsItem);

		sprintf(szString, "Material Index: %08lX", m_vMaterials[i].m_iIndex);

		InsertItem(hTreeCtrl, 0, szString, hMaterialItem);

		sprintf(szString, "Used By: %i face(s)", m_vMaterials[i].m_iTris);

		InsertItem(hTreeCtrl, 0, szString, hMaterialItem);
	}

	// *** aggiunge la lista di nodi ***

	DumpInTreeCtrl(hTreeCtrl);
}

//==========================================================================================