// TimeSlider.cpp : implementation file
//

#include "stdafx.h"
#include "MODELER.h"
#include "TimeSlider.h"

#include "MODELERDoc.h"
#include "MODELERView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTimeSlider

CTimeSlider::CTimeSlider() :
	m_pAnimOpenGLRenderer(NULL),
	m_uiHeight(18),
	m_iButtonWidth(16), m_iThumbWidth(125), m_iSliderWidth(m_iButtonWidth*2+m_iThumbWidth),
	m_iFrameMapWidth(0),
	m_pMODELERView(NULL)
{
}

CTimeSlider::~CTimeSlider()
{
}


BEGIN_MESSAGE_MAP(CTimeSlider, CWnd)
	//{{AFX_MSG_MAP(CTimeSlider)
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
	ON_COMMAND(2, OnPrevFrameButton)
	ON_COMMAND(3, OnNextFrameButton)
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTimeSlider message handlers

BOOL CTimeSlider::OnEraseBkgnd(CDC* pDC) 
{
	CRect rectClient;
	GetClientRect (rectClient);
	pDC->FillSolidRect (&rectClient, RGB(225,225,225));

	return TRUE;
}

int CTimeSlider::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// *** crea i buttons ***

	CRect rectDummy (0, 0, 0, 0);

	DWORD dwButtonStyle = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;

	if ( m_wndPrevFrame.Create("", dwButtonStyle, rectDummy, this, (UINT)2) == 0 ||
		m_wndNextFrame.Create("", dwButtonStyle, rectDummy, this, (UINT)3) == 0 ||
		m_wndThumb.Create("", dwButtonStyle, rectDummy, this, (UINT)4) == 0 )
	{
		TRACE0("Failed to create children in CTimeSlider::OnCreate!");
		return -1;
	}

	m_wndPrevFrame.SetImage(theApp.LoadIcon(IDI_PREVFRAME));
	m_wndNextFrame.SetImage(theApp.LoadIcon(IDI_NEXTFRAME));

	m_wndThumb.SetFont(&globalData.fontBold);

	m_wndPrevFrame.SetTooltip("Go to the previous frame");
	m_wndNextFrame.SetTooltip("Go to the next frame");
	m_wndThumb.SetTooltip("Drag the time slider to set the current frame");

	return 0;
}

void CTimeSlider::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);

	RecalcLayout(cx, cy);
}

void CTimeSlider::RecalcLayout (int cx, int cy)
{
	m_iSliderPos = 0;

	if ( m_pAnimOpenGLRenderer == NULL || m_pAnimOpenGLRenderer->m_pSceneRoot == NULL )
	{
		if ( m_wndThumb.GetStyle() & WS_VISIBLE )
		{
			m_wndPrevFrame.ShowWindow(SW_HIDE);
			m_wndThumb.ShowWindow(SW_HIDE);
			m_wndNextFrame.ShowWindow(SW_HIDE);
		}
	}
	else
	{
		if ( (m_wndThumb.GetStyle() & WS_VISIBLE) == 0 )
		{
			m_wndPrevFrame.ShowWindow(SW_SHOW);
			m_wndThumb.ShowWindow(SW_SHOW);
			m_wndNextFrame.ShowWindow(SW_SHOW);
		}

		m_iFrameMapWidth = cx - m_iSliderWidth + 1;

		if ( m_iFrameMapWidth <= 0 )
		{
			m_iFrameMapWidth = 0;
		}
		else
		{

			int				iTime = m_pAnimOpenGLRenderer->m_tTime / m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;
			int				iStart = m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->first_frame;
			int				iEnd = m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->last_frame;

			float			fFrameNumber = (float)iStart;
			float			fDelta = (float)(iEnd - iStart) / m_iFrameMapWidth;

			int				iIndex = m_iFrameMapWidth-1;

			for ( int i=0; i<m_iFrameMapWidth-1; i++ )
			{
				m_iFrameMap[i] = (int)fFrameNumber;
				if ( iIndex == m_iFrameMapWidth-1 && m_iFrameMap[i] >= iTime )
				{
					iIndex = i;
				}
				fFrameNumber += fDelta;
			}

			m_iFrameMap[i] = iEnd;

			m_iSliderPos = iIndex;

		}

		CRect		rectWindow;
		m_wndPrevFrame.GetWindowRect(&rectWindow);

		if ( rectWindow.Width() == 0 )
		{

			m_wndPrevFrame.SetWindowPos(NULL, m_iSliderPos, 0, m_iButtonWidth, m_uiHeight, SWP_NOACTIVATE | SWP_NOZORDER);
			m_wndThumb.SetWindowPos(NULL, m_iSliderPos + m_iButtonWidth, 0, m_iThumbWidth, m_uiHeight, SWP_NOACTIVATE | SWP_NOZORDER);
			m_wndNextFrame.SetWindowPos(NULL, m_iSliderPos + m_iButtonWidth + m_iThumbWidth, 0, m_iButtonWidth, m_uiHeight, SWP_NOACTIVATE | SWP_NOZORDER);

		}
		else
		{

			ScreenToClient(&rectWindow);

			ScrollWindow(m_iSliderPos-rectWindow.left, 0);

		}

		// *** imposta il testo del thumb ***

		char		szString[MAX_PATH];

		sprintf(szString, "%i / %i", m_pAnimOpenGLRenderer->m_tTime / m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame,
			m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->last_frame - m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->first_frame);

		m_wndThumb.SetWindowText(szString);
	}
}

void CTimeSlider::OnPrevFrameButton()
{
	// *** ricava la posizione del puntatore del mouse ***

	CPoint		pntRelativeCursorPos;

	GetCursorPos(&pntRelativeCursorPos);

	m_wndPrevFrame.ScreenToClient(&pntRelativeCursorPos);

	// *** aggiorna il time del renderer ***

	m_pAnimOpenGLRenderer->m_tTime -= m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;

	if ( m_pAnimOpenGLRenderer->m_tTime < m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->first_frame * m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame )
	{
		m_pAnimOpenGLRenderer->m_tTime = m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->last_frame * m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;
	}

	m_pAnimOpenGLRenderer->RedrawWindow();

	CRect		rectClient;
	GetClientRect(&rectClient);

	RecalcLayout(rectClient.Width(), rectClient.Height());

	// *** aggiorna la posizione del puntatore del mouse ***

	CRect		rectWindow;

	m_wndPrevFrame.GetWindowRect(&rectWindow);

	CPoint		pntAbsoluteCursorPos(rectWindow.left + pntRelativeCursorPos.x, rectWindow.top + pntRelativeCursorPos.y);

	SetCursorPos(pntAbsoluteCursorPos.x, pntAbsoluteCursorPos.y);

	// *** si occupa di una eventuale view attaccata ***

	if ( m_pMODELERView )
	{
		m_pMODELERView->m_bStopButtonEnabled = TRUE;
		m_pMODELERView->StopAnimation();
	}
}

void CTimeSlider::OnNextFrameButton()
{
	// *** ricava la posizione del puntatore del mouse ***

	CPoint		pntRelativeCursorPos;

	GetCursorPos(&pntRelativeCursorPos);

	m_wndNextFrame.ScreenToClient(&pntRelativeCursorPos);

	// *** aggiorna il time del renderer ***

	m_pAnimOpenGLRenderer->m_tTime += m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;

	if ( m_pAnimOpenGLRenderer->m_tTime > m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->last_frame * m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame )
	{
		m_pAnimOpenGLRenderer->m_tTime = m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->first_frame * m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;
	}

	m_pAnimOpenGLRenderer->RedrawWindow();

	CRect		rectClient;
	GetClientRect(&rectClient);

	RecalcLayout(rectClient.Width(), rectClient.Height());

	// *** aggiorna la posizione del puntatore del mouse ***

	CRect		rectWindow;

	m_wndNextFrame.GetWindowRect(&rectWindow);

	CPoint		pntAbsoluteCursorPos(rectWindow.left + pntRelativeCursorPos.x, rectWindow.top + pntRelativeCursorPos.y);

	SetCursorPos(pntAbsoluteCursorPos.x, pntAbsoluteCursorPos.y);

	// *** si occupa di una eventuale view attaccata ***

	if ( m_pMODELERView )
	{
		m_pMODELERView->m_bStopButtonEnabled = TRUE;
		m_pMODELERView->StopAnimation();
	}
}

BOOL CTimeSlider::PreTranslateMessage(MSG* pMsg)
{
	if ( pMsg->message == WM_LBUTTONDOWN &&
		pMsg->hwnd == m_wndThumb.m_hWnd )
	{
		CPoint		pntCursorPos;
		GetCursorPos(&pntCursorPos);

		CRect		rectWindow;
		m_wndPrevFrame.GetWindowRect(&rectWindow);

		m_iOffset = pntCursorPos.x - rectWindow.left;

		SetFocus();

		SetCapture();

		// *** si occupa di una eventuale view attaccata ***

		if ( m_pMODELERView )
		{
			m_pMODELERView->StopAnimation();
		}

		return TRUE;
	}

	return CWnd::PreTranslateMessage(pMsg);
}

void CTimeSlider::OnMouseMove(UINT nFlags, CPoint point)
{
	if ( GetCapture() == this )
	{
		int			iProvSliderPos = point.x - m_iOffset;

		if ( iProvSliderPos < 0 )
		{
			iProvSliderPos = 0;
		}
		else if ( iProvSliderPos > m_iFrameMapWidth-1 )
		{
			iProvSliderPos = m_iFrameMapWidth-1;
		}

		int			iNewFrame = m_iFrameMap[iProvSliderPos];

		if ( iNewFrame != m_pAnimOpenGLRenderer->m_tTime / m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame )
		{
			m_pAnimOpenGLRenderer->m_tTime = iNewFrame * m_pAnimOpenGLRenderer->m_pSceneRoot->m_pGI->ticks_per_frame;

			m_pAnimOpenGLRenderer->RedrawWindow();

			CRect		rectClient;
			GetClientRect(&rectClient);

			RecalcLayout(rectClient.Width(), rectClient.Height());

			// *** si occupa di una eventuale view attaccata ***

			if ( m_pMODELERView )
			{
				m_pMODELERView->m_bStopButtonEnabled = TRUE;
			}
		}
	}

	CWnd::OnMouseMove(nFlags, point);
}

void CTimeSlider::OnLButtonUp(UINT nFlags, CPoint point)
{
	if ( GetCapture() == this )
		ReleaseCapture();
	
	CWnd::OnLButtonUp(nFlags, point);
}

void CTimeSlider::OnContextMenu(CWnd*, CPoint point)
{
	return;
}